import java.io.*; 
import java.util.Date;

class EX13F
{
    private Date dateLastUpdated;   // Date the last time the stats for this program were written.
    private String userLastUpdated; // User's name that performed the last update.
    private int executionCount;     // The number of times this program has been run.

    // This method reads the program stats from a file by the same name as the class.
    public void retrieveStats() throws IOException
    {
        try
        {
            // Create a data stream filter with a file input stream as the host stream.
            DataInputStream dis = new DataInputStream(new FileInputStream(getClass().getName() + ".dat"));

            // Initialize the data members with the values in the stat file.
            dateLastUpdated = new Date(dis.readLong());
            userLastUpdated = dis.readUTF();
            executionCount = dis.readInt();
        }
        catch (FileNotFoundException e) 
        {
            // Eat this exception since the file won't exist initially.
        }
    }

    // This method writes the program stats back out to the same file.
    public void saveStats() throws IOException
    {
        // Update date is now.
        Date today = new Date();

        // This time create an output data filter attached to a file output stream.
        DataOutputStream dos = new DataOutputStream(new FileOutputStream(getClass().getName() + ".dat"));

        // Write each value back out to the file.
        dos.writeLong(Date.parse(today.toGMTString()));
        if (userLastUpdated.length() == 0)
            userLastUpdated = "Ace Programmer";
        dos.writeUTF(userLastUpdated);
        dos.writeInt(++executionCount);
    }

    public static void main(String args[])
    {
        EX13F example = new EX13F();  // Create an instance.

        try 
        {   // Read the stats in from the last run.
            example.retrieveStats();

            // And print them out.
            System.out.println("Program last update date: " + example.dateLastUpdated);
            System.out.println("Last updated by: " + example.userLastUpdated);
            System.out.println("Execution count: " + example.executionCount);

            // Now prompt the user for their name to update the stat file.
            System.out.print("\nWhat is your name: ");
            System.out.flush();

            // Use a data stream again so we can use readLine() 
            // to read from the console.
            DataInputStream stdin = new DataInputStream(System.in);
            example.userLastUpdated = stdin.readLine();

            // Finally, save the stats back out to the file.
            example.saveStats();
        } 
        catch (Exception e) 
        {
            System.err.println("Error processing stats file: " + e);
        }
    }
}
