import java.applet.*;
import java.awt.*;
import java.io.*;

public class EX13G extends Applet implements Runnable
{
    private Thread m_Ex13g = null;
    private Graphics m_Graphics;
    private Image m_Images[];
    private int m_nCurrImage = 0;
    private int m_nImgWidth  = 0;
    private int m_nImgHeight = 0;
    private boolean m_fAllLoaded = false;
    private final int NUM_IMAGES = 18;

    private PipedInputStream inputPipe;
    private PipeMonitor monitor;

    public EX13G()
    {   // Create input pipe and share with pipe monitor.
        inputPipe = new PipedInputStream();
        monitor = new PipeMonitor(inputPipe);
    }

    public void start()
    {   // Start applet in its own thread.
        if (m_Ex13g == null)
        {
            m_Ex13g = new Thread(this);
            m_Ex13g.start();
        }
    }

    public void stop()
    {   // Stop applet and pipe monitor threads.
        if (m_Ex13g != null)
        {
            m_Ex13g.stop();
            monitor.stop();
            m_Ex13g = null;
        }
    }

    public void run()
    {
        if (!m_fAllLoaded)
        {   // Load all images first.
            if (!loadImages())
                return;
        }

        while (true)
        {   // Check to see if globe should be animated.
            if (monitor.isAnimated())
            {
                try
                {   // Display image and move on to next image.
                    displayImage(m_Graphics);
                    if (monitor.getRotation() == monitor.FORWARD)
                    {   // Move forward through image array.
                        if (++m_nCurrImage == NUM_IMAGES)
                            m_nCurrImage = 0;
                    }
                    else
                    {   // Move backwards through image array.
                        if (--m_nCurrImage < 0)
                            m_nCurrImage = NUM_IMAGES - 1;
                    }               

                    // Rate of animation is controlled by sleep time.
                    Thread.currentThread().sleep(monitor.getSleepTime());
                }
                catch (InterruptedException e)
                {
                    stop();
                    monitor.stop();
                }
            }
        }
    }

    private boolean loadImages()
    {
        boolean retval = true;

        // Standard image loading logic created by Applet Wizard.
        repaint();
        m_Graphics = getGraphics();
        m_Images   = new Image[NUM_IMAGES];

        MediaTracker tracker = new MediaTracker(this);
        String strImage;

        for (int i = 1; i <= NUM_IMAGES; i++)
        {
            strImage = "images/img00" + ((i < 10) ? "0" : "") + i + ".gif";
            m_Images[i-1] = getImage(getDocumentBase(), strImage);
            tracker.addImage(m_Images[i-1], 0);
        }

        try
        {
            tracker.waitForAll();
            m_fAllLoaded = !tracker.isErrorAny();
        }
        catch (InterruptedException e) {}
			
        if (!m_fAllLoaded)
        {
            stop();
            m_Graphics.drawString("Error loading images!", 10, 20);
            retval = false;
        }
        else
        {
            m_nImgWidth  = m_Images[0].getWidth(this);
            m_nImgHeight = m_Images[0].getHeight(this);
            repaint();
        }
        return retval;
    }

    private void displayImage(Graphics g)
    {   // Draw image on center of component.
        if (m_fAllLoaded)
        {
            g.drawImage(m_Images[m_nCurrImage], (size().width - m_nImgWidth) / 2,
                    (size().height - m_nImgHeight) / 2, null);
        }
    }

    public void paint(Graphics g)
    {   // Display current image or message.
        if (m_fAllLoaded)
        {
            Rectangle r = g.getClipRect();
            g.clearRect(r.x, r.y, r.width, r.height);
            displayImage(g);
        }
        else
            g.drawString("Loading images...", 10, 20);
    }

    public void connectPipes(PipedOutputStream out) throws IOException
    {   // Called by controller applet to connect input & output pipes.
        inputPipe.connect(out);
        // Only start pipe monitor if necessary.
        if (!monitor.isAlive())
            monitor.start();    // Waits for commands sent through pipe.
    }
}

class PipeMonitor extends Thread
{
    public final int FORWARD = 1;
    public final int REVERSE = 2;

    // Setup attributes with default values.
    private int rotation = FORWARD;
    private int sleepTime = 50;
    private boolean animated = true;
    private DataInputStream inputPipe;

    private PipeMonitor() {} // Hide default ctor.

    public PipeMonitor(PipedInputStream pi) 
    { 
        inputPipe = new DataInputStream(pi); 
    }

    // Access methods.
    public synchronized int getRotation() { return rotation; }
    public synchronized int getSleepTime() { return sleepTime; }
    public synchronized boolean isAnimated() { return animated; }

    public void run()
    {
        String cmd;
        while (true)
        {
            try
            {   // Wait for a ('\n' terminated) command from the pipe.
                do
                    cmd = inputPipe.readLine();
                while (cmd == null);

                // Interpret command and change settings.
                if (cmd.equalsIgnoreCase("FORWARD"))
                    rotation = FORWARD;
                else if (cmd.equalsIgnoreCase("REVERSE"))
                    rotation = REVERSE;
                else if (cmd.equalsIgnoreCase("START"))
                    animated = true;
                else if (cmd.equalsIgnoreCase("STOP"))
                    animated = false;
                else if (cmd.equalsIgnoreCase("FASTER"))
                    sleepTime -= (sleepTime == 10 ? 0 : 10);
                else if (cmd.equalsIgnoreCase("SLOWER"))
                    sleepTime += 10;
            }
            catch (IOException e) {}
        }
    }
}
